<div class="pagination-centred">
  <ul class="pagination">
    {% for item in data %}
    <li{% for class in item.0 %}{% if forloop.first %} class="{% endif %}{{ class }}{% if not forloop.last %} {% endif %}{% if forloop.last %}"{% endif %}{% endfor %}>
      <a href="{{ item.1 }}"{% if item.3 %} title="{{ item.3 }}"{% endif %}>{{ item.2|safe }}</a>
    </li>
    {% endfor %}
  </ul>
</div>
